/* If you hold down ALT, you can make a move in your opponent's turn. */

#include <SDL.h>
#include <string.h>
#include <signal.h>
#include "sdlhelp.h"
#include "blockit.h"
#include "game.h"
#include "rungame.h"
#include "blockitnet.h"

int bl_ugly = 0;

SDL_Surface *bl_screen;

SDL_Surface *bl_bmp_title;
SDL_Surface *bl_bmp_sidebar_button, *bl_bmp_sidebar_large, *bl_bmp_sidebar_small;
SDL_Surface *bl_bmp_win_red, *bl_bmp_win_blue, *bl_bmp_win_none;
SDL_Surface *bl_bmp_scroll_bar,*bl_bmp_scroll_up,*bl_bmp_scroll_down,*bl_bmp_scroll_nib;
SDL_Surface *bl_bmp_blocks[8]; /* Terra{neutral,red,blue,shared} Stone{//} */
SDL_Surface *bl_bmp_bblocks[8];/* Terra{neutral,red,blue,shared} Stone{//} */

static int fullscreen = SDL_FULLSCREEN;

/* From the SDL manual, adapted for Blockit: toggle between fullscreen & normal modes. */
static int blockit_toggle_fullscreen(const SDL_Event *event)
{
  SDL_Surface *screenbuf;

  if (event->type == SDL_QUIT)
    {
      SDL_Quit();
      exit(0);
    }

  if (event->type == SDL_KEYDOWN)
    {
      if ( (event->key.keysym.sym == SDLK_RETURN) &&
	   (event->key.keysym.mod & KMOD_ALT) )
	{
	  
	  screenbuf = SDL_CreateRGBSurface(SDL_SWSURFACE, bl_screen->w, bl_screen->h, 16,
					   bl_screen->format->Rmask,
					   bl_screen->format->Gmask,
					   bl_screen->format->Gmask,
					   bl_screen->format->Amask);
	  SDL_BlitSurface(bl_screen, NULL, screenbuf, NULL);
	  
      SDL_WM_SetCaption("Block It", NULL);
	  
	  if (fullscreen == SDL_FULLSCREEN) fullscreen != 0;
	  else fullscreen = SDL_FULLSCREEN;
	  
	  if((bl_screen = SDL_SetVideoMode(640, 480, 16, SDL_SWSURFACE|fullscreen)) == NULL)
	    {
	      SDLhelp_Error("Could not toggle between windowed and fullscreen modes.\n");
	    }

	  SDL_BlitSurface(screenbuf, NULL, bl_screen, NULL);
	  SDL_FreeSurface(screenbuf);
	  SDL_UpdateRect(bl_screen, 0, 0, 0, 0);
	  return 0;
	}
    }
  return 1;
}

int main(int argc, char **argv)
{
  char *argv0, *exepath;
  int black, white;

  if (argc != 1) /* This is not necessary. */
    {
      if ((argc != 2))
	SDLhelp_Error("usage: blockit [window]\n\nTo cheat in multiplayer, ... read the source for main() in blockit.c !");
      if ((strcasecmp(argv[1], "window")==0))
	fullscreen = 0;
    }

  if(SDL_Init(SDL_INIT_VIDEO) < 0)
    {
      SDLhelp_Error("Could not initialise DirectMedia.");
    }
  atexit(SDL_Quit);

  SDL_WM_SetCaption("Blockit", NULL);
  if((bl_screen = SDL_SetVideoMode(640, 480, 16, SDL_SWSURFACE)) == NULL)
    {
      SDLhelp_Error("Could not get a 640x480x16 graphics mode.\n");
    }
  if (SDL_MUSTLOCK(bl_screen)) SDL_Quit(); /* Die. Shouldn't happen, though. */


  black = SDL_MapRGB(bl_screen->format, 0x00, 0x00, 0x00);
  white = SDL_MapRGB(bl_screen->format, 0xFF, 0xFF, 0xFF);
  SDL_FillRect(bl_screen, NULL, black);
  SDLhelp_DrawString(bl_screen, 280, 232, "Loading...", white);
  SDL_UpdateRect(bl_screen, 0, 0, 0, 0);
  SDLhelp_InitTicks();

  argv0 = strdup(argv[0]);
  if (argv0 == NULL)
    {
      SDLhelp_Problem(bl_screen, "Out of memory!");
      SDL_Quit();
      return 1;
    }
  exepath= SDLhelp_LocateExecutable(argv0);
  id_load_bitmaps(exepath);
  free(argv0);

  bl_convert_bitmaps();
  network_init();


  SDL_SetEventFilter(blockit_toggle_fullscreen);

  bl_title_screen();

  network_shutdown();
  SDL_Quit(); /* ... programme stops here ... */
  return 0;
}

static void bl_draw_screen(int redraw, char *label1, char *label2, char *label3, char *label4)
{
  Uint32 black, white;
  SDL_Rect re;
  SDL_Surface *buf1, *buf2;

  black = SDL_MapRGB(bl_screen->format, 0x00, 0x00, 0x00);
  white = SDL_MapRGB(bl_screen->format, 0xFF, 0xFF, 0xFF);

  if (redraw == 2)
    {
      buf1 = SDL_CreateRGBSurface(0, 640, 480, 16,
				  bl_screen->format->Rmask, bl_screen->format->Gmask,
				  bl_screen->format->Bmask, bl_screen->format->Amask);
      SDL_BlitSurface(bl_screen, NULL, buf1, NULL);
    }
  SDL_BlitSurface(bl_bmp_title, NULL, bl_screen, NULL);
  re.x = 160;
  re.w = 320;
  re.h = 40;
  re.y = 200;
  SDL_BlitSurface(bl_bmp_sidebar_button, NULL, bl_screen, &re);
  re.y = 250;
  SDL_BlitSurface(bl_bmp_sidebar_button, NULL, bl_screen, &re);
  re.y = 300;
  SDL_BlitSurface(bl_bmp_sidebar_button, NULL, bl_screen, &re);
  re.y = 350;
  SDL_BlitSurface(bl_bmp_sidebar_button, NULL, bl_screen, &re);
  SDLhelp_DrawString(bl_screen, 320-strlen(label1)*4, 212, label1, black);
  SDLhelp_DrawString(bl_screen, 320-strlen(label2)*4, 262, label2, black);
  SDLhelp_DrawString(bl_screen, 320-strlen(label3)*4, 312, label3, black);
  SDLhelp_DrawString(bl_screen, 321-strlen(label4)*4, 363, label4, white);
  SDLhelp_DrawString(bl_screen, 320-strlen(label4)*4, 362, label4, black);
  
  if (redraw == 2)
    {
      buf2 = SDL_CreateRGBSurface(0, 640, 480, 16,
				  bl_screen->format->Rmask, bl_screen->format->Gmask,
				  bl_screen->format->Bmask, bl_screen->format->Amask);
      SDL_BlitSurface(bl_screen, NULL, buf2, NULL);
      
      SDL_BlitSurface(buf1, NULL, bl_screen, NULL);
      SDL_UpdateRect(bl_screen, 0, 0, 0, 0);
      
      SDLhelp_CrossFade(bl_screen, buf2, 640, 480);
      
      SDL_FreeSurface(buf1);
      SDL_FreeSurface(buf2);
    }
  else
    {
      if(redraw)
	SDL_UpdateRect(bl_screen, 0, 0, 0, 0);
    }
}

/* Draw and handle events for the main title screen. */
void bl_title_screen()
{
  Uint32 black, white;
  SDL_Event ev;
  int quitflag = 0;
  int redraw = 2;
  int x;

  black = SDL_MapRGB(bl_screen->format, 0x00, 0x00, 0x00);
  white = SDL_MapRGB(bl_screen->format, 0xFF, 0xFF, 0xFF);

  for (;;)
    {
      if (redraw)
	{
	  bl_draw_screen(redraw, "One-player game", "Two-player game", "Options", "Quit");
	  redraw = 0;
	}
      
      SDL_WaitEvent(&ev);
      if (ev.type == SDL_QUIT) quitflag = 1;
      if (ev.type == SDL_KEYUP) /* ESC key ==> Quit */
	{
	  if(ev.key.keysym.sym == SDLK_ESCAPE)
	    quitflag = 1;
	}
      if (ev.type == SDL_MOUSEBUTTONDOWN)
	{
	  if ((ev.button.x >= 160) && (ev.button.x <= 480)) /* Correct X coord */
	    {
	      if ((ev.button.y >= 200) && (ev.button.y <= 240)) /* 1-player */
		{
		  redraw = 1;
		  x = bl_select_1p();
		  if (x == -1) continue;
		  if (x == R_AI_HARD)
		    r_start_game(R_HUMAN, x);
		  else
		    r_start_game(x, R_HUMAN);
		  redraw = 2;
		}
	      if ((ev.button.y >= 250) && (ev.button.y <= 290)) /* 2-player */
		{
		  if (network_okay)
		    {
		      redraw = 1;
		      x = bl_select_2p();
		      if (x == -1) continue;
		    }
		  else
		    {
		      x = R_HUMAN;
		    }
		  if (x < R_NET_MASTER)
		    r_start_game(R_HUMAN, x);
		  else
		    r_connect_net(x);
		  redraw = 2;
		}
	      if ((ev.button.y >= 300) && (ev.button.y <= 340)) /* Options */
		{
		  bl_select_options();
		  redraw = 1;
		}
	      if ((ev.button.y >= 350) && (ev.button.y <= 390)) /* Quit */
		{
		  quitflag = 1;
		}
	    }
	}

      if (quitflag) break;
    }
  SDLhelp_DissolveScreen(bl_screen, 640, 480, black);
}

void bl_select_options()
{
  SDL_Event ev;
  int x = -1;
  int redraw = 1;
  Uint32 black, white;

  black = SDL_MapRGB(bl_screen->format, 0x00, 0x00, 0x00);
  white = SDL_MapRGB(bl_screen->format, 0xFF, 0xFF, 0xFF);
  
  for (;;)
    {
      if (redraw)
	{
	  bl_draw_screen(0, "Large board", "Small board", "Help", "Okay");
	  if (G_WIDTH == 24)
	    SDLhelp_DrawString(bl_screen, 212, 180, "Board size is 24x16 (large)", white);
	  else
	    SDLhelp_DrawString(bl_screen, 212, 180, "Board size is 15x10 (small)", white);
	  SDL_UpdateRect(bl_screen, 0, 0, 0, 0);
	  redraw = 0;
	}
      SDL_WaitEvent(&ev);
      if (ev.type == SDL_QUIT) SDL_Quit();
      if (ev.type == SDL_KEYUP)
	if (ev.key.keysym.sym == SDLK_ESCAPE)
	    break;
      if (ev.type == SDL_MOUSEBUTTONDOWN)
	{
	  if((ev.button.x >= 160) && (ev.button.x <= 480)) /* Check X */
	    {
	      if ((ev.button.y >= 200) && (ev.button.y <= 240)) /* Large board */
		{
		  G_WIDTH = 24; G_HEIGHT = 16;
		  redraw = 1;
		}
	      if ((ev.button.y >= 250) && (ev.button.y <= 290)) /* Small board */
		{
		  G_WIDTH = 15; G_HEIGHT = 10;
		  redraw = 1;
		}
	      if ((ev.button.y >= 300) && (ev.button.y <= 340)) /* Help */
		{
		  h_help();
		  redraw = 1;
		}
	      if ((ev.button.y >= 350) && (ev.button.y <= 390)) /* Quit */
		{
		  break;
		}
	    }
	}
    }
}

int  bl_select_1p()
{
  SDL_Event ev;
  int x = -1;

  bl_draw_screen(1, "Easy computer", "Normal computer", "Hard computer", "Cancel");
  for (;;)
    {
      SDL_WaitEvent(&ev);
      if (ev.type == SDL_QUIT) SDL_Quit();
      if (ev.type == SDL_KEYUP)
	if (ev.key.keysym.sym == SDLK_ESCAPE)
	    break;
      if (ev.type == SDL_MOUSEBUTTONDOWN)
	{
	  if((ev.button.x >= 160) && (ev.button.x <= 480)) /* Check X */
	    {
	      if ((ev.button.y >= 200) && (ev.button.y <= 240)) /* Easy */
		{
		  x = R_AI_EASY;
		  break;
		}
	      if ((ev.button.y >= 250) && (ev.button.y <= 290)) /* Normal */
		{
		  x = R_AI_NORMAL;
		  break;
		}
	      if ((ev.button.y >= 300) && (ev.button.y <= 340)) /* Hard */
		{
		  x = R_AI_HARD;
		  break;
		}
	      if ((ev.button.y >= 350) && (ev.button.y <= 390)) /* Quit */
		{
		  break;
		}
	    }
	}
    }
  return x;
}

int bl_select_2p()
{
  SDL_Event ev;
  int x = -1;

  bl_draw_screen(1, "Normal game", "Start a network game", "Join a network game", "Cancel");
  for (;;)
    {
      SDL_WaitEvent(&ev);
      if (ev.type == SDL_QUIT) SDL_Quit();
      if (ev.type == SDL_KEYUP)
	if (ev.key.keysym.sym == SDLK_ESCAPE)
	    break;
      if (ev.type == SDL_MOUSEBUTTONDOWN)
	{
	  if((ev.button.x >= 160) && (ev.button.x <= 480)) /* Check X */
	    {
	      if ((ev.button.y >= 200) && (ev.button.y <= 240)) /* Human */
		{
		  x = R_HUMAN;
		  break;
		}
	      if ((ev.button.y >= 250) && (ev.button.y <= 290)) /* Network master */
		{
		  x = R_NET_MASTER;
		  break;
		}
	      if ((ev.button.y >= 300) && (ev.button.y <= 340)) /* Network slave */
		{
		  x = R_NET_SLAVE;
		  break;
		}
	      if ((ev.button.y >= 350) && (ev.button.y <= 390)) /* Quit */
		{
		  break;
		}
	    }
	}
    }
  return x;
}

void bl_convert_bitmaps()
{
  SDL_Surface *tmp;
  int i;

  tmp = SDL_DisplayFormat(bl_bmp_title);
  SDL_FreeSurface(bl_bmp_title);
  bl_bmp_title = tmp;

  tmp = SDL_DisplayFormat(bl_bmp_sidebar_button);
  SDL_FreeSurface(bl_bmp_sidebar_button);
  bl_bmp_sidebar_button = tmp;

  tmp = SDL_DisplayFormat(bl_bmp_sidebar_large);
  SDL_FreeSurface(bl_bmp_sidebar_large);
  bl_bmp_sidebar_large = tmp;

  tmp = SDL_DisplayFormat(bl_bmp_sidebar_small);
  SDL_FreeSurface(bl_bmp_sidebar_small);
  bl_bmp_sidebar_small = tmp;

  for (i=0; i<6; i++)
    {
      tmp = SDL_DisplayFormat(bl_bmp_blocks[i]);
      SDL_FreeSurface(bl_bmp_blocks[i]);
      bl_bmp_blocks[i] = tmp;
    }

  tmp = SDL_DisplayFormat(bl_bmp_win_red);
  SDL_FreeSurface(bl_bmp_win_red);
  bl_bmp_win_red = tmp;

  tmp = SDL_DisplayFormat(bl_bmp_win_blue);
  SDL_FreeSurface(bl_bmp_win_blue);
  bl_bmp_win_blue = tmp;

  tmp = SDL_DisplayFormat(bl_bmp_win_none);
  SDL_FreeSurface(bl_bmp_win_none);
  bl_bmp_win_none = tmp;
}
