#include <stdio.h>
#include <SDL.h>
#include "blockit.h"
#include "sdlhelp.h"
#include "game.h"

int G_WIDTH = 15, G_HEIGHT = 10;
char g_grid[24][16];

/* Set grid to initial state */
void g_start_game()
{
  int i,j;

  memset (g_grid, 0, sizeof(g_grid));
}

int g_can_move(int player, int x, int y)
{
  if ((x < 0)||(x>=G_WIDTH)||(y < 0)||(y>=G_HEIGHT)) return 0;
  if (g_grid[x][y] & GT_STONE) return 0;
  if (g_grid[x][y] & player) return 1;
  if (g_grid[x][y] == 0) return 1;
  return 0;
}

/* Place a stone at the given location. */
void g_make_move(int player, int x, int y)
{
  int i, j;

  if (player == GP_RED)
    {
      g_grid[x][y] = GP_RED  | GT_STONE;
    }
  else if (player == GP_BLUE)
    {
      g_grid[x][y] = GP_BLUE | GT_STONE;
    }
  else
    {
      SDLhelp_Problem(bl_screen, "Bad player in g_make_move()");
    }

  for(i=x-1; i<=x+1; i++)
    {
      if (i<0) continue;
      if (i>=G_WIDTH) continue;
      for (j=y-1; j<=y+1; j++)
	{
	  if (j<0) continue;
	  if (j>=G_HEIGHT) continue;
	  if (g_grid[i][j] & GT_STONE)
	    {
	      if ((player == GP_RED) &&
		  (g_grid[i][j] & GP_BLUE))
		{
		  g_grid[i][j] = GP_RED | GT_STONE;
		}
	      if ((player == GP_BLUE) &&
		  (g_grid[i][j] & GP_RED))
		{
		  g_grid[i][j] = GP_BLUE | GT_STONE;
		}
	    }
	}
    }
  g_compute_territory();
}

/* Work out which squares are owned by which player. */
void g_compute_territory()
{
  int i, j;
  int x, y;
  int red_own, blue_own;

  for (i=0; i<G_WIDTH; i++)
    {
      for (j=0; j<G_HEIGHT; j++)
	{
	  if (g_grid[i][j] & GT_STONE) continue;
	  red_own = blue_own = 0;
	  for (x=i-1; x<=i+1; x++)
	    {
	      if (x<0) continue;
	      if (x>=G_WIDTH) continue;
	      for (y=j-1; y<=j+1; y++)
		{
		  if (y<0) continue;
		  if (y>=G_HEIGHT) continue;
		  if (g_grid[x][y] & GT_STONE)
		    {
		      if (g_grid[x][y] & GP_RED) red_own =1;
		      if (g_grid[x][y] & GP_BLUE) blue_own =1;
		    }
		}
	    }
	  if ((red_own == 0) && (blue_own == 0)) g_grid[i][j] = GP_NEUTRAL;
	  if ((red_own == 1) && (blue_own == 0)) g_grid[i][j] = GP_RED;
	  if ((red_own == 0) && (blue_own == 1)) g_grid[i][j] = GP_BLUE;
	  if ((red_own == 1) && (blue_own == 1)) g_grid[i][j] = GP_SHARED;
	}
    }
}

/* Count the number of squares and sort them by colour. */
void g_get_territory(int *neutral, int *red, int *blue)
{
  int i, j;
  int n=0, r=0, b=0;

  for (i=0; i<G_WIDTH; i++)
    {
      for (j=0; j<G_HEIGHT; j++)
	{
	  if ((g_grid[i][j] & 3)==GP_RED) r++;
	  else if ((g_grid[i][j] & 3)==GP_BLUE) b++;
	  else n++;
	}
    }
  *neutral = n;
  *red = r;
  *blue = b;
}

void g_get_stones(int *red, int *blue)
{
  int i, j;
  int r=0, b=0;

  for (i=0; i<G_WIDTH; i++)
    {
      for (j=0; j<G_HEIGHT; j++)
	{
	  if ((g_grid[i][j] & GT_STONE) && (g_grid[i][j] & GP_RED)) r++;
	  if ((g_grid[i][j] & GT_STONE) && (g_grid[i][j] & GP_BLUE)) b++;
	}
    }
  *red = r;
  *blue = b;
}

int  g_has_finished()
{
  int sr, sb, winner, over = 0;
  int i, j, x, y;
  int bcm=0, rcm=0;
  int colour;

  g_get_territory(&winner, &sr, &sb);

  /* GAME OVER CHECK.
   *
   * 1. If either player can't move, the game is finished.
   * 2. If all squares are occupied and there are no unoccupied squares
   *    adjacent to enemy squares, the game is finished.
   *
   */
  /* Check #1 */
  for (i=0; i<G_WIDTH; i++)
    {
      for (j=0; j<G_HEIGHT; j++)
	{
	  if (g_can_move(GP_RED, i, j)) rcm ++;
	  if (g_can_move(GP_BLUE,i, j)) bcm ++;
	}
    }
  if ((!rcm)||(!bcm)) /* Neither can move */
    {
      over = 1;
    }

  /* Check #2 */
  if ((sr+sb)==(G_WIDTH*G_HEIGHT)) over = 1; /* All squares occupied */
  for (i=0; i<G_WIDTH; i++)
    {
      for (j=0; j<G_HEIGHT; j++)
	{
	  if (g_grid[i][j] & GT_STONE) continue;
	  colour = g_grid[i][j];
	  for (x=i-1; x<=i+1; x++)
	    {
	      if (x < 0) continue;
	      if (x >= G_WIDTH) continue;
	      for (y=j-1; y<=j+1; y++)
		{
		  if (y < 0) continue;
		  if (y >= G_HEIGHT) continue;

		  if (g_grid[x][y] & GT_STONE) continue;
		  if (g_grid[x][y] != colour) return 0;
		}
	    }
	}
    }

  winner = 0;
  if (over)
    {
      if (sr > sb) winner = GP_RED;
      if (sb > sr) winner = GP_BLUE;
      if (winner == 0) winner = GP_SHARED; /* No winner */
    }

  return over? winner:over;
}

