#include <SDL.h>
#include <string.h>
#include "sdlhelp.h"
#include "blockit.h"

static int H_LINES = -1;

typedef struct
{
  char *text;
  int highlighted;
} help_line;

help_line blockit_help[] =
{
  /*[XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX]*/
  { "Help for Blockit                                                   v1.2", 1 },
  { "Game concept and programming by Cameron Patrick            January 2001", 0 },
  { "Email: cp@chem.com.au                       WWW: http://chem.com.au/cp/", 0 },
  { "", 0 },
  { "", 0 },
  { "HOW TO PLAY BLOCKIT", 1 },
  { "Blockit is a 2-player computerised \"board game\", in which the aim is", 0 },
  { "to occupy as many squares of the board (territory) as possible.", 0 },
  { "Initially, all of the board is unoccupied. Each player takes turns in", 0 },
  { "placing down \"stones\" (pieces) on the board. Stones can't be placed", 0 },
  { "on squares which already have stones on, or on enemy territory.", 0 },
  { "", 0 },
  { "Territory", 1 },
  { "Each square is said to belong to either the red player, the blue player", 0 },
  { "or to no-one (neutral territory). Squares immediately surrounding red", 0 },
  { "stones are red territory and squares immediately surrounding blue", 0 },
  { "stones are blue territory. If a square doesn't touch any stone, it is", 0 },
  { "neutral territory. If a square touches both red and blue stones, it", 0 },
  { "is contested territory, and is coloured yellow to signify that a move", 0 },
  { "here will let you capture enemy pieces.", 0 },
  { "", 0 },
  { "Taking enemy pieces", 1 },
  { "When you place a stone down, all enemy stones immediately surrounding", 0 },
  { "it becomes yours.", 0 },
  { "", 0 },
  { "", 0 },
  { "MULTI-PLAYER GAMES", 1 },
  { "Blockit allows two humans to play against each other in two ways:", 0 },
  { "two people taking turns at one machine, or two people on two different", 0 },
  { "computers connected by a network---either the Internet, or a smaller", 0 },
  { "network.", 0 },
  { "", 0 },
  { "Local-area networks (i.e., using a network card)", 1 },
  { "Each person should start up Blockit. Select a two-player game.", 0 },
  { "If you are running Windows and no menu appears, then you don't have", 0 },
  { "TCP/IP networking set up. Setting this up involves installing drivers", 0 },
  { "using the control panel and giving each machine an IP address, which", 0 },
  { "is too much to explain here.", 0 },
  { "", 0 },
  { "One person should select \"Start a network game\" from the 2-player", 0 },
  { "game menu. Then, the second person selects \"Join a network game\"", 0 },
  { "from the menu. When you are asked for your opponent's IP address,", 0 },
  { "get it from the screen of the person who started the game.", 0 },
  { "If the address is correct and the network is properly set up, the", 0 },
  { "game should be ready to start in a few seconds.", 0 },
  { "", 0 },
  { "It is also possible to talk to the other player by typing while the", 0 },
  { "game is in progress.  Remember to press ENTER after you have finished", 0 },
  { "typing, or the message won't be sent.", 0 },
  { "", 0 },
  { "Playing over the Internet", 1 },
  { "Blockit will work in exactly the same way over the Internet. The", 0 },
  { "most important difference is that you will have to get the IP address", 0 },
  { "of your opponent over the phone/email/IRC/whatever first. (Hint: it", 0 },
  { "may be simpler to do this if you turn off full-screen mode by pressing", 0 },
  { "Alt-Enter. This will allow you to have your mail/chat programme open at", 0 },
  { "the same time as Blockit.)", 0 },
  { "", 0 },
  { "If you are not connected to the Internet when you start Blockit,", 0 },
  { "the 2-player network options will not be available.", 0 },
  { "", 0 },
  { "-- IMPORTANT NOTE ABOUT INTERNET PLAY --  The chances are that your IP", 0 },
  { "address will be different each time that you connect to the Internet.", 0 },
  { "", 0 },
  { "", 0 },
  { "SINGLE-PLAYER GAMES", 1 },
  { "Blockit has three built-in computer opponents of varying difficulty.", 0 },
  { "", 0 },
  { "In easy mode, the computer picks random squares to move into, unless", 0 },
  { "it can attack. If it can attack, it picks a random yellow square to", 0 },
  { "move into.", 0 },
  { "", 0 },
  { "In normal mode, the computer picks random squares to move into, but", 0 },
  { "it tries to avoid your squarees. If it can attack, it chooses the", 0 },
  { "move that will gain it the most pieces in the least time.", 0 },
  { "", 0 },
  { "In hard mode, the computer has pre-programmed openings that it will", 0 },
  { "choose randomly. It also looks ahead a move when it makes attacks.", 0 },
  { "If hard mode isn't hard enough, there is a separate programme which", 0 },
  { "has more complex strategies built-in; it communicates with Blockit", 0 },
  { "using the network mode.", 0 },
  { "", 0 },
  { "", 0 },
  { "VARIABLE BOARD SIZE", 0 },
  { "Blockit allows two different sized boards: large (24x16) and", 0 },
  { "small (15x10). A smaller board means shorter and simpler games.", 0 },
  { "A larger board allows longer games with more strategy. By default,", 0 },
  { "Blockit uses a small board, but if you feel as if it is getting too", 0 },
  { "easy or boring, you may want to try the larger size.", 0 },
  { "", 0 },
  { "", 0 },
  { "", 0 },
  { "END OF TEXT", 0 },
  { NULL, -1 }
};

static void redraw_help(int top_line)
{
  int black, white, red, dred;
  SDL_Rect re;
  int i;

  black = SDL_MapRGB(bl_screen->format, 0x00, 0x00, 0x00);
  white = SDL_MapRGB(bl_screen->format, 0xFF, 0xFF, 0xFF);
  red   = SDL_MapRGB(bl_screen->format, 0xFF, 0x80, 0x00);
  dred  = SDL_MapRGB(bl_screen->format, 0x80, 0x00, 0x00);

  r_fade_screen (0.5, 0, 0);

  re.x = 0; re.y = 0; re.w = 512; re.h = 32;
  SDL_BlitSurface(bl_bmp_sidebar_large, NULL, bl_screen, &re);
  re.x = 512; re.y = 0; re.w = 128; re.h = 32;
  SDL_BlitSurface(bl_bmp_sidebar_small, NULL, bl_screen, &re);
  SDLhelp_DrawString(bl_screen, 556, 8, "Close", black);
  SDLhelp_DrawString(bl_screen, 16, 8, "Blockit help viewer", black);

  re.x = 600; re.y = 40; re.w = 16; re.h = 16;
  SDL_BlitSurface(bl_bmp_scroll_up, NULL, bl_screen, &re);
  re.x = 600; re.y = 56; re.w = 16; re.h = 384;
  SDL_BlitSurface(bl_bmp_scroll_bar, NULL, bl_screen, &re);
  re.x = 600; re.y = 440; re.w = 16; re.h = 16;
  SDL_BlitSurface(bl_bmp_scroll_down, NULL, bl_screen, &re);
  re.x = 600; re.y = 56 + ((384*top_line)/(H_LINES-1)); re.w = 16; re.h = 16;
  if (re.y > 424) re.y = 424;
  SDL_BlitSurface(bl_bmp_scroll_nib, NULL, bl_screen, &re);

#if 1
  for (i=0; i<5; i++)
    {
      if (blockit_help[i].highlighted)
	{
	  SDLhelp_DrawString(bl_screen, 11, 43+i*16, blockit_help[i].text, black);
	  SDLhelp_DrawString(bl_screen, 10, 42+i*16, blockit_help[i].text, dred);
	  SDLhelp_DrawString(bl_screen, 9, 41+i*16, blockit_help[i].text, red);
	}
      else
	{
	  SDLhelp_DrawString(bl_screen, 9, 41+i*16, blockit_help[i].text, black);
	}
      SDLhelp_DrawString(bl_screen, 8, 40+i*16, blockit_help[i].text, white);
    }
#endif

#if 0
  for (i=top_line; i<(top_line+25); i++)
#else
  for (i=(top_line+5); i<(top_line+25); i++)
#endif
    {
      if (i > (H_LINES-1))
	{
	  break;
	}
      if (blockit_help[i].highlighted)
	{
	  SDLhelp_DrawString(bl_screen, 11, 43+i*16-top_line*16, blockit_help[i].text, black);
	  SDLhelp_DrawString(bl_screen, 10, 42+i*16-top_line*16, blockit_help[i].text, dred);
	  SDLhelp_DrawString(bl_screen, 9, 41+i*16-top_line*16, blockit_help[i].text, red);
	}
      else
	{
	  SDLhelp_DrawString(bl_screen, 9, 41+i*16-top_line*16, blockit_help[i].text, black);
	}
      SDLhelp_DrawString(bl_screen, 8, 40+i*16-top_line*16, blockit_help[i].text, white);
    }

  /* Kludge for drawing a straight line!! Yuck. */
  SDLhelp_DrawString(bl_screen, 8, 440, "________________________________________________________________________", white);
  SDLhelp_DrawString(bl_screen, 9, 440, "________________________________________________________________________", white);
  SDLhelp_DrawString(bl_screen, 8, 456, "Use the cursor keys to scroll, or drag the scrollbar with the mouse.", white);

  SDL_UpdateRect(bl_screen, 0, 0, 0, 0);
}

void h_help()
{
  int line = 0;
  int redraw = 1;
  int mouse_is_down = 0;
  int x;
  SDL_Event ev;

  if (H_LINES == -1)
    {
      for (H_LINES=0; blockit_help[H_LINES].text != NULL; H_LINES++);
    }

  for(;;)
    {
      if (redraw)
	{
	  redraw_help (line);
	  redraw = 0;
	}
      SDL_WaitEvent(&ev);
      if (ev.type == SDL_QUIT) break;
      if (ev.type == SDL_MOUSEBUTTONDOWN)
	{
	  if ((ev.button.x >= 512) && (ev.button.y <= 32)) return;
	  if ((ev.button.x >= 600)&&(ev.button.y <= 616))
	    {
	      if ((ev.button.y >= 40)&&(ev.button.y <= 56))
		{
		  ev.type = SDL_KEYDOWN; ev.key.keysym.sym = SDLK_UP;
		}
	      if ((ev.button.y >= 440)&&(ev.button.y <= 456))
		{
		  ev.type = SDL_KEYDOWN; ev.key.keysym.sym = SDLK_DOWN;
		}
	      if ((ev.button.y >= 56)&&(ev.button.y <= 440))
		{
		  mouse_is_down = 1;
		}
	    }
	}
      if ((ev.type == SDL_MOUSEMOTION) && mouse_is_down)
	{
	  x = ((ev.motion.y - 56) * (H_LINES-1)) / 384;
	  if (x<0) x = 0;
	  if (x>(H_LINES-2)) x = H_LINES-1;
	  if (x != line)
	    {
	      line = x; redraw = 1;
	    }
	}
      if (ev.type == SDL_MOUSEBUTTONUP)
	{
	  mouse_is_down = 0;
	}
      if (ev.type == SDL_KEYDOWN)
	{
	  if ((ev.key.keysym.sym == SDLK_DOWN))
	    {
	      line++;
	      if (line > (H_LINES - 2)) line = H_LINES - 1;
	      redraw = 1;
	    }
	  if ((ev.key.keysym.sym == SDLK_UP) && (line > 0))
	    {
	      line--;
	      redraw = 1;
	    }
	  if ((ev.key.keysym.sym == SDLK_PAGEDOWN))
	    {
	      line+=10;
	      if (line > (H_LINES - 2)) line = H_LINES - 1;
	      redraw = 1;
	    }
	  if ((ev.key.keysym.sym == SDLK_PAGEUP))
	    {
	      line-=10;
	      if (line < 0) line = 0;
	      redraw = 1;
	    }
	}
      if (ev.type == SDL_KEYUP)
	{
	  if (ev.key.keysym.sym == SDLK_ESCAPE) return;
	}
    }

}
