#include <stdio.h>
#include <malloc.h>
#include <zlib.h>
#include <SDL.h>
#include "blockit.h"
#include "imagedata.h"

/* If there is no BLOCKIT.CID file, we create our own ugly-looking
 * bitmaps
 */

static SDL_Surface *new_bitmap(int x, int y)
{
  SDL_Surface *im;

  im = SDL_CreateRGBSurface(SDL_SWSURFACE, x, y, 16,
			    bl_screen->format->Rmask,
			    bl_screen->format->Gmask,
			    bl_screen->format->Bmask,
			    bl_screen->format->Amask);
  return im;
}

static SDL_Surface *generate_win_message(char *m, int r, int g, int b)
{
  int bg, fg;
  SDL_Surface *im;

  im = new_bitmap(320, 40);

  bg = SDL_MapRGB(im->format, 0, 0, 0);
  fg = SDL_MapRGB(im->format, r, g, b);

  SDL_FillRect(im, NULL, bg);
  SDLhelp_DrawString(im, 160-strlen(m)*4, 12, m, fg);

  return im;
}

static SDL_Surface *generate_sidebar(int x, int y)
{
  int black, white, bg;
  SDL_Surface *im;
  SDL_Rect r;

  im = new_bitmap(x, y);
  black = SDL_MapRGB(im->format, 0, 0, 0);
  white = SDL_MapRGB(im->format, 0xFF, 0xFF, 0xFF);
  bg = SDL_MapRGB(im->format, 0x40, 0xC0, 0xFF);
  SDL_FillRect(im, NULL, white);
  r.x=1;  r.y=1;  r.w = x-2; r.h = y-2;
  SDL_FillRect(im, &r, black);
  r.x=2;  r.y=2;  r.w = x-4; r.h = y-4;
  SDL_FillRect(im, &r, bg);

  return im;
}

static SDL_Surface *generate_arrow(int x, int y, int dir)
{
  int black, white, bg;
  SDL_Surface *im;
  SDL_Rect r;
  int i;

  im = new_bitmap(x, y);
  black = SDL_MapRGB(im->format, 0, 0, 0);
  white = SDL_MapRGB(im->format, 0xFF, 0xFF, 0xFF);
  bg = SDL_MapRGB(im->format, 0x40, 0xC0, 0xFF);
  SDL_FillRect(im, NULL, white);
  r.x=1;  r.y=1;  r.w = x-2; r.h = y-2;
  SDL_FillRect(im, &r, black);
  r.x=2;  r.y=2;  r.w = x-4; r.h = y-4;
  SDL_FillRect(im, &r, bg);

  if (dir)
    {
      r.x = 4; r.y = 4; r.w = x-8; r.h = 2;
      SDL_FillRect(im, &r, black);
    }
  else
    {
      r.x = 4; r.y = y-5; r.w = x-8; r.h = 2;
      SDL_FillRect(im, &r, black);
    }

  return im;
}

static SDL_Surface *generate_block(int x, int y, int stone, int colour)
{
  int black, white, lt, dk;
  SDL_Surface *im;
  SDL_Rect r;
  int i;

  im = new_bitmap(x, y);

  black = SDL_MapRGB(im->format, 0, 0, 0);
  white = SDL_MapRGB(im->format, 0xFF, 0xFF, 0xFF);

  switch (colour)
    {
    case 0: /* grey */
      lt = SDL_MapRGB(im->format, 0xC0, 0xC0, 0xC0);
      dk = SDL_MapRGB(im->format, 0x80, 0x80, 0x80);
      break;
    case 1: /* red */
      lt  = SDL_MapRGB(im->format, 0xFF, 0x80, 0x20);
      dk  = SDL_MapRGB(im->format, 0x80, 0x00, 0x00);
      break;
    case 2: /* blue */
      lt = SDL_MapRGB(im->format, 0x40, 0xC0, 0xFF);
      dk = SDL_MapRGB(im->format, 0x00, 0x00, 0x80);
      break;
    case 3: /* yellow */
      lt  = SDL_MapRGB(im->format, 0xFF, 0xFF, 0x00);
      dk  = SDL_MapRGB(im->format, 0x80, 0x80, 0x00);
      break;
    }

  SDL_FillRect(im, NULL, black);
  r.x=1;  r.y=1;  r.w = x-2; r.h = y-2;
  SDL_FillRect(im, &r, lt);
  if (stone)
    {
      r.x=2;  r.y=2;  r.w = x-4; r.h = y-4;
      SDL_FillRect(im, &r, dk);
    }

  return im;
}

static void generate_ugly_bitmaps()
{
  int i;
  Uint32 dkblue, white, black;
  SDL_PixelFormat *fmt;

  /* Title */
  bl_bmp_title = new_bitmap(640,480);
  
  fmt = bl_bmp_title->format;
  black = SDL_MapRGB(fmt, 0x00, 0x00, 0x00);
  white = SDL_MapRGB(fmt, 0xFF, 0xFF, 0xFF);
  dkblue= SDL_MapRGB(fmt, 0x08, 0x10, 0x70);
  
  SDL_FillRect(bl_bmp_title, NULL, dkblue);
  SDLhelp_DrawString(bl_bmp_title, 321-28, 41, "BLOCKIT", black);
  SDLhelp_DrawString(bl_bmp_title, 320-28, 40, "BLOCKIT", white);
  SDLhelp_DrawString(bl_bmp_title, 8, 460, "by Cameron Patrick", white);
  SDLhelp_DrawString(bl_bmp_title, 640-160, 460, "Missing blockit.cid", white);

  /* Red/Blue/No-one wins */
  bl_bmp_win_none = generate_win_message("NO WINNER", 0x00, 0xFF, 0x00);
  bl_bmp_win_blue = generate_win_message("BLUE WINS", 0x00, 0x00, 0xFF);
  bl_bmp_win_red  = generate_win_message("RED WINS",  0xFF, 0x00, 0x00);

  /* Side bar */
  bl_bmp_sidebar_large = generate_sidebar(512,32);
  bl_bmp_sidebar_small = generate_sidebar(128,32);
  bl_bmp_sidebar_button = generate_sidebar(320, 40);

  /* Scroll bar */
  bl_bmp_scroll_bar = generate_sidebar(16,384);
  bl_bmp_scroll_nib = generate_sidebar(16,16);
  bl_bmp_scroll_up = generate_arrow(16,16,1);
  bl_bmp_scroll_down = generate_arrow(16,16,0);

  /* Blocks */
  for (i=0; i<8; i++)
    {
      bl_bmp_blocks[i]  = generate_block(20, 20, i&4, i&3);
      bl_bmp_bblocks[i] = generate_block(32, 32, i&4, i&3);
    }

}

void id_load_bitmaps(char *path)
{
  int i;
  gzFile *f;
  char *buf;
  int w, h;
  SDL_Surface *im;
  unsigned char *pixbuf;
  int x, y;

  buf = malloc(strlen(path) + 80);
  sprintf(buf,  "%s/blockit.cid", path);
  f = gzopen(buf, "rb");
  if (f == NULL)
    {
      generate_ugly_bitmaps();
      bl_ugly = 1;
      return;
    }
  for (i=0; i<ID_N_BITMAPS; i++)
    {
      w = id_bitmaps[i].width;
      h = id_bitmaps[i].height;
      im = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, 24,
				0x000000FF, 0x0000FF00, 0x00FF0000, 0);
      pixbuf = malloc(w*h*3);
      if (pixbuf == NULL)
	{
	  SDLhelp_Problem(bl_screen, "Out of memory!");
	  SDL_Quit();
	  exit(1);
	}
      gzread(f,pixbuf,w*h*3);
      for (y=0; y<im->h; y++)
	{
	  for (x=0; x<im->w; x++)
	    {
	      ((char*)im->pixels)[(y*im->pitch)+x*3]   = pixbuf[(y*w*3)+x*3];
	      ((char*)im->pixels)[(y*im->pitch)+x*3+1] = pixbuf[(y*w*3)+x*3+1];
	      ((char*)im->pixels)[(y*im->pitch)+x*3+2] = pixbuf[(y*w*3)+x*3+2];
	    }
	}
      free(pixbuf);
      *id_bitmap_ptrs[i] = im;
    }
  free(buf);
  gzclose(f);
}

