/* Handles the drawing/UI side of network mode. All prototypes are in
 * "rungame.h", not netgame.h; ha!
 */
#include <SDL.h>
#include "sdlhelp.h"
#include "game.h"
#include "blockit.h"
#include "blockitnet.h"
#include "rungame.h"

/* Special `moves': (166, 166) ==> stop the game
                    (180, 180) ==> start talk mode */

typedef struct
{
  char text[80]; /* Limited to 70 in reality */
  int player; /* GP_RED or GP_BLUE */
} talkline;

static talkline talkline_save[25];
static int talk_is_okay = 0;

static void redraw_connect_screen(int net_type, int mode, char *addr, int port)
{
  SDL_Rect re;
  int black, white, grey;
  char buf[80];

  black = SDL_MapRGB(bl_screen->format, 0x00, 0x00, 0x00);
  white = SDL_MapRGB(bl_screen->format, 0xFF, 0xFF, 0xFF);
  grey  = SDL_MapRGB(bl_screen->format, 0xA0, 0xA0, 0xA0);

  /* Draw up the screen */
  SDL_BlitSurface(bl_bmp_title, NULL, bl_screen, NULL);
  /* r_fade_screen(0, .5, .5); */
  
  re.x = 0; re.y = 0; re.w = 512; re.h = 32;
  SDL_BlitSurface(bl_bmp_sidebar_large, NULL, bl_screen, &re);
  re.x = 512; re.y = 0; re.w = 128; re.h = 32;
  SDL_BlitSurface(bl_bmp_sidebar_small, NULL, bl_screen, &re);

  if (mode < 4)
    {
      SDLhelp_DrawString(bl_screen, 522, 8, "ESC to cancel", black);
    }
  else
    {
      SDLhelp_DrawString(bl_screen, 534, 8, "Disconnect", black);
    }
    
  if (mode < 4)
    {
      if (net_type == R_NET_MASTER)
	{
	  if (mode == 0)
	    SDLhelp_DrawString(bl_screen, 8, 8, "Waiting for connection...", black);
	  else
	    SDLhelp_DrawString(bl_screen, 8, 8, "Connecting to other player...", black);
	}
      else
	{
	  if (mode == 0)
	    SDLhelp_DrawString(bl_screen, 8, 8, "Type your opponent's IP address", black);
	  if (mode >= 1)
	    SDLhelp_DrawString(bl_screen, 8, 8, "Connecting to other player...", black);
	}
    }
  else
    {
      SDLhelp_DrawString(bl_screen, 8, 8, "Connection established.", black);
    }

  SDLhelp_DrawString(bl_screen, 8, 240, "Opponent's IP address:", white);
  if ((net_type == R_NET_SLAVE))
    {
      SDLhelp_DrawString(bl_screen, 552, 240, "RED player", white);
      SDLhelp_DrawString(bl_screen, 544, 320, "BLUE player", white);
      SDLhelp_DrawString(bl_screen, 32, 256, addr, white);
      if (mode == 0)
	SDLhelp_DrawString(bl_screen, 32+strlen(addr)*8, 256, "_", white);
      if (mode == 1)
	SDLhelp_DrawString(bl_screen, 8, 200, "Network: Looking up host...", white);
      if (mode == 2)
	SDLhelp_DrawString(bl_screen, 8, 200, "Network: Host contacted, waiting for reply.", white);
      if ((mode >= 3)&&(mode!=666))
	SDLhelp_DrawString(bl_screen, 8, 200, "Network: Connection established.", white);
      if (mode==666)
	SDLhelp_DrawString(bl_screen, 8, 200, "Network: Connection out of sync, trying to re-establish.", white);
    }
  else
    {
      SDLhelp_DrawString(bl_screen, 544, 240, "BLUE player", white);
      SDLhelp_DrawString(bl_screen, 552, 320, "RED player", white);
      switch(mode)
	{
	case 0:
	  SDLhelp_DrawString(bl_screen, 8, 200, "Network: Waiting for other player to join...", white);
	  SDLhelp_DrawString(bl_screen, 32, 256, "unknown", white);
	  break;
	case 1:
	  SDLhelp_DrawString(bl_screen, 8, 200, "Network: Host contacted, waiting for reply.", white);
	  SDLhelp_DrawString(bl_screen, 32, 256, addr, white);
	  break;
	case 666:
	  SDLhelp_DrawString(bl_screen, 8, 200, "Network: Connection out of sync, trying to re-establish.", white);
	  SDLhelp_DrawString(bl_screen, 32, 256, addr, white);
	  break;
	default:
	  SDLhelp_DrawString(bl_screen, 8, 200, "Network: Connection established.", white);
	  SDLhelp_DrawString(bl_screen, 32, 256, addr, white);
	  break;
	}
    }

  if (mode >= 3)
    {
      SDLhelp_DrawString(bl_screen, 320-40*4, 360, "Press F4 to enter talk mode at any time.", white);
      re.x = 0; re.y = 400; re.w = 320; re.h = 40;
      SDL_BlitSurface(bl_bmp_sidebar_button, NULL, bl_screen, &re);
      re.x = 320; re.y = 400; re.w = 320; re.h = 40;
      SDL_BlitSurface(bl_bmp_sidebar_button, NULL, bl_screen, &re);
      SDLhelp_DrawString(bl_screen, 68, 412, "Start game - large board", black);
      SDLhelp_DrawString(bl_screen, 388, 412, "Start game - small board", black);
    }

  SDLhelp_DrawString(bl_screen, 8, 320, "Your IP address:", white);
  SDLhelp_DrawString(bl_screen, 32, 336, network_ip_buffer, white);

  if (port != 0)
    {
      sprintf(buf, "Port %d", port);
      SDLhelp_DrawString(bl_screen, 632-strlen(buf)*8, 200, buf, white);
    }

  SDL_UpdateRect(bl_screen, 0, 0, 0, 0);
}

static void r_network_error(char *problem)
{
  SDL_Surface *screenbuf;
  SDL_Rect re;
  SDL_Event ev;
  int black, white, grey;

  black = SDL_MapRGB(bl_screen->format, 0x00, 0x00, 0x00);
  white = SDL_MapRGB(bl_screen->format, 0xFF, 0xFF, 0xFF);
  grey  = SDL_MapRGB(bl_screen->format, 0xA0, 0xA0, 0xA0);

  screenbuf = SDL_CreateRGBSurface(SDL_SWSURFACE, bl_screen->w, bl_screen->h, 16,
				   bl_screen->format->Rmask,
				   bl_screen->format->Gmask,
				   bl_screen->format->Gmask,
				   bl_screen->format->Amask);
  SDL_BlitSurface(bl_screen, NULL, screenbuf, NULL);

  /* Draw up the screen */
  SDL_BlitSurface(bl_bmp_title, NULL, bl_screen, NULL);

  SDLhelp_DrawString(bl_screen, 269, 200, "NETWORK ERROR", black);
  SDLhelp_DrawString(bl_screen, 268, 201, "NETWORK ERROR", white);
  SDLhelp_DrawString(bl_screen, 320-strlen(problem)*4, 280, problem, white);

  re.x = 160; re.y = 400; re.w = 320; re.h = 40;
  SDL_BlitSurface(bl_bmp_sidebar_button, NULL, bl_screen, &re);
  SDLhelp_DrawString(bl_screen, 320-32, 412, "Continue", black);
  
  SDL_UpdateRect(bl_screen, 0, 0, 0, 0);

  for (;;)
    {
      if (SDL_PollEvent(&ev))
	{
	  if ((ev.type == SDL_KEYUP)||(ev.type==SDL_MOUSEBUTTONDOWN)||(ev.type==SDL_QUIT))
	    break;
	}
    }
  
  SDL_BlitSurface(screenbuf, NULL, bl_screen, NULL);
  SDL_FreeSurface(screenbuf);
  SDL_UpdateRect(bl_screen, 0, 0, 0, 0);
}

int r_connect_net(int net_type)
{
  SDL_Event ev;
  char buf[80];
  int m, x, y, tmp;
  int port = 0;
  int redraw = 0;
  int startgame = 0, startchat = 0, in_sync = 1;

  network_kill_connection();

  // kludge - try to connect to Localhost first, before attempting
  // to connect to the world at large.  This makes life easier for
  // Blockit_AI.
  if (net_type == R_NET_SLAVE)
    {
      strcpy(buf, "localhost");
      if ((port = network_client_connect(buf)) == -1)
	{
	  goto main_connect;
	}
      redraw_connect_screen(R_NET_SLAVE, 2, buf, port);
      if (network_client_handshake() == -1)
	{
	  goto main_connect;
	}
      goto done_connect;
    }

 main_connect:
  strcpy (buf, "");
  redraw_connect_screen(net_type, 0, buf, 0);
  network_kill_connection();
  if (net_type == R_NET_SLAVE)
    {
      for (;;)
	{
	  SDL_WaitEvent(&ev);
	  if ((ev.type == SDL_QUIT))
	    {
	      return -1;
	    }
	  if (ev.type == SDL_KEYUP)
	    {
	      if(ev.key.keysym.sym == SDLK_ESCAPE)
		{
		  return -1;
		}
	      if(ev.key.keysym.sym == SDLK_RETURN)
		{
		  break;
		}
	    }
	  if ((ev.type == SDL_KEYDOWN))
	    {
	      if(ev.key.keysym.sym == SDLK_BACKSPACE)
		{
		  if (strlen(buf) > 0) buf[strlen(buf)-1] = '\0';
		  redraw_connect_screen(net_type, 0, buf, 0);
		  continue;
		}
	      if ((strlen(buf) < 32)&&(isprint(ev.key.keysym.sym)))
		{
		  x = strlen(buf);
		  buf[x] = ev.key.keysym.sym;
		  buf[x+1] = '\0';
		  redraw_connect_screen(net_type, 0, buf, 0);
		}
	    }
	}
      redraw_connect_screen(R_NET_SLAVE, 1, buf, 0);
      if ((port = network_client_connect(buf)) == -1)
	{
	  r_network_error("Couldn't connect to remote machine.");
	  return -1;
	}
      redraw_connect_screen(R_NET_SLAVE, 2, buf, port);
      if (network_client_handshake() == -1)
	{
	  r_network_error("Remote machine doesn't seem to be running Blockit.");
	  return -1;
	}
    }
  else
    {
      if ((port = network_start_server()) == -1)
	{
	  r_network_error("Couldn't start Blockit server.");
	  return -1;
	}
      redraw_connect_screen(net_type, 0, buf, port);
      for (;;)
	{
	  if(SDL_PollEvent(&ev))
	    {
	      if ((ev.type == SDL_KEYUP)  &&  (ev.key.keysym.sym == SDLK_ESCAPE))
		{
		  return -1;
		}
	      if ((ev.type == SDL_QUIT))
		{
		  return -1;
		}
	    }
	  x = network_check_connect();
	  if (x == 1)
	    {
	      strcpy(buf, network_get_enemy_ip());
	      redraw_connect_screen(net_type, 1, buf, port);
	      if (network_server_handshake() == -1)
		{
		  r_network_error("A connection was received, but refused to identify itself.");
		  strcpy(buf, "");
		  redraw_connect_screen(net_type, 0, buf, port);
		  return -1;
		}
	      break;
	    }
	  if (x == -1)
	    {
	      r_network_error("Connection has died.");
	      return -1;
	    }
	}
    }

 done_connect:
  /* Connected! Yay! */

  /* ********************************************************** */

  redraw = 1;
  r_chat_mode(-1); /* Erase previous messages. */
  SDL_StartTicks();

  for (;;)
    {
      if (redraw)
	{
	  if (in_sync)
	    redraw_connect_screen(net_type, 15, buf, port);
	  else
	    redraw_connect_screen(net_type,666, buf, port);
	  redraw = 0;
	}
      if (SDL_PollEvent(&ev))
	{
	  if ((ev.type==SDL_MOUSEBUTTONDOWN) && in_sync)
	    {
	      if (ev.button.y <= 40)
		break;
	      if (ev.button.y >= 400)
		{
		  if (ev.button.x < 320)
		    {
		      G_WIDTH = 24; G_HEIGHT = 16; /* Start game/large board */
		      network_send_move(META_STARTLARGE, 0, 0);
		      startgame = 1;
		    }
		  if (ev.button.x > 320)
		    {
		      G_WIDTH = 15; G_HEIGHT = 10; /* Start game/small board */
		      network_send_move(META_STARTSMALL, 0, 0);
		      startgame = 2;
		    }
		}
	    }
	  if (ev.type == SDL_KEYUP)
	    {
	      if (ev.key.keysym.sym == SDLK_ESCAPE)
		{
		  break;
		}
	      if ((ev.key.keysym.sym == SDLK_F4) && in_sync)
		{
		  network_send_move(META_CHATMODE, 0, 0);
		  startchat = 1;
		}
	    }
	}
      if ((!in_sync)&&((SDL_GetTicks()%100)==0))
	{
	  x = network_synchronise();
	  if (x==-1) break; /* Connection lost */
	  if (x==1)
	    {
	      in_sync = 1; redraw = 1;
	      continue;
	    }
	}
      if (network_check_for_move() && in_sync)
	{
	  
	  tmp = network_get_move(&m, &x, &y);
	  if (tmp==-1) break;
	  if (tmp==-2)
	    {
	      in_sync = 0; redraw = 1; continue;
	    }
	  if ((m==META_STARTLARGE))
	    startgame = 1;
	  else if ((m==META_STARTSMALL))
	    startgame = 2;
	  else if ((m==META_CHATMODE))
	    startchat = 1;
	  else
	    {
	      in_sync = 0; redraw = 1; continue;
	    }
	}
      if (startchat)
	{
	  r_chat_mode(net_type);
	  startchat = 0; redraw = 1;
	}
      if (startgame)
	{
	  if (startgame == 2)
	    {
	      G_WIDTH = 15; G_HEIGHT = 10;
	    }
	  else
	    {
	      G_WIDTH = 24; G_HEIGHT = 16;
	    }
	  r_start_game(R_HUMAN, net_type);
	  startgame = 0; redraw = 1;
	}
    }

  network_kill_connection();
  return 0;
}

static void draw_chatmode(talkline *tb, char *buf, int m)
{
  int white, blue, red;
  int i;

  white = SDL_MapRGB(bl_screen->format, 0xFF, 0xFF, 0xFF);
  blue  = SDL_MapRGB(bl_screen->format, 0x00, 0x00, 0x80);
  red   = SDL_MapRGB(bl_screen->format, 0x80, 0x00, 0x00);

  r_fade_screen(0, 0.5, 0.5);

  SDLhelp_DrawString(bl_screen, 9, 9, "BLOCKIT TALK MODE - press F4 to toggle between chat and game", blue);
  SDLhelp_DrawString(bl_screen, 8, 8, "BLOCKIT TALK MODE - press F4 to toggle between chat and game", white);

  SDLhelp_DrawString(bl_screen, 9, 441, buf, (m == R_NET_MASTER)?red:blue);
  SDLhelp_DrawString(bl_screen, 8, 440, buf, white);
  SDLhelp_DrawString(bl_screen, 8+strlen(buf)*8, 440, "_", white);

  for (i=0; i<25; i++)
    {
      SDLhelp_DrawString(bl_screen, 9, 41+i*16, tb[i].text, (tb[i].player == GP_RED)?red:blue);
      SDLhelp_DrawString(bl_screen, 8, 40+i*16, tb[i].text, white);
    }

  SDL_UpdateRect(bl_screen, 0, 0, 0, 0);
}

static void add_text_line(char *buf, talkline *tb, int player)
{
  int i;

  for (i=1; i<25; i++)
    {
      strcpy(tb[i-1].text, tb[i].text);
      tb[i-1].player = tb[i].player;
    }
  strcpy(tb[24].text, buf);
  tb[24].player = player;
}

int r_chat_mode(int type)
{
  char buf[80];
  char inbuf[80];
  int redraw = 1;
  SDL_Event ev;
  int x, y;
  talkline *text_buffer = talkline_save;
  int i, c;

  strcpy(buf, "");
  if (type == -1) talk_is_okay = 0;
  if (!talk_is_okay)
    {
      for (i=0; i<25; i++)
	{
	  strcpy(text_buffer[i].text, "");
	  text_buffer[i].player = GP_RED;
	}
      talk_is_okay = 1;
    }
  if (type == -1) return 0;
  
  for (;;)
    {
      if (redraw) {
	draw_chatmode(text_buffer, buf, type);
	redraw = 0;
      }
      if (network_check_for_move())
	{
	  x = network_get_text(inbuf);
	  if (x==-1) return -1;
	  if (x==1) return 0;
	  add_text_line(inbuf, text_buffer, (type == R_NET_MASTER)?GP_BLUE:GP_RED);
	  redraw = 1;
	}
      if (SDL_PollEvent(&ev))
	{
	  if (ev.type == SDL_QUIT)
	    {
	      network_end_talk();
	      return 0;
	    }
	  if ((ev.type == SDL_KEYUP))
	    {
	      if ((ev.key.keysym.sym == SDLK_F4) || (ev.key.keysym.sym == SDLK_ESCAPE))
		{
		  network_end_talk();
		  return 0;
		}
	      if (ev.key.keysym.sym == SDLK_RETURN)
		{
		  if (strlen(buf) == 0) continue;
		  network_send_text(buf);
		  add_text_line(buf, text_buffer, (type == R_NET_MASTER)?GP_RED:GP_BLUE);
		  strcpy(buf, "");
		  redraw = 1;
		}
	    }
	  if (ev.type == SDL_KEYDOWN)
	    {
	      if (ev.key.keysym.sym == SDLK_BACKSPACE)
		{
		  if (strlen(buf) > 0)
		    {
		      buf[strlen(buf)-1] = '\0';
		    }
		  redraw = 1;
		}
	      if (!isprint(ev.key.keysym.sym) || (ev.key.keysym.sym > 255)) continue;
	      c = r_shifted_keystroke(ev.key.keysym.sym);
	      if (strlen(buf) < 70)
		{
		  x = strlen(buf);
		  buf[x] = c;
		  buf[x+1] = '\0';
		  redraw = 1;
		}
	    }
	}
    }
}

