/* network.c - For multiplayer blockit games. The only file (other than SDLhelp)
   that depends on WinXX vs Unix differences.
*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <SDL.h>
#include "blockitnet.h"

#define MAGIC_1 "Bl.Ver1.1"
#define MAGIC_2 "Blockit!!"
#define MAGIC_3 "MOVE==>"
/* MAGIC_4:  There is _no_ magic 4. */
#define MAGIC_5 "===>"
#define MAGIC_6 "<==="

#define N_BLOCKIT_PORTS 3
static unsigned blockit_port[3] =
{
  5678,
  8765,
  6886
};

/* Get the nasty stuff over and done with. */
#ifdef WIN32
#include <winsock.h>

#define BAD_SOCK INVALID_SOCKET

#else

#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <inet.h>

#define BAD_SOCK -1
#define SOCKET int
#define closesocket close
#endif

int network_okay = 0;
static SOCKET server_socket = 0;
static SOCKET game_socket;
static fd_set socks;



#ifdef WIN32
void network_init()
{
  WSADATA temp;

  network_okay = 1;
  if(WSAStartup(MAKEWORD(1,1), &temp) != 0)
    {
      network_okay = 0;
    }
  if (network_okay)
    {
      if (network_get_local_ip() < 0) network_okay = 0;
    }
}

void network_shutdown()
{
  if (WSACleanup() < 0)
    {
      WSACancelBlockingCall();
      WSACleanup();
    }
  network_okay = 0;
}

#else
void network_init()
{
  network_okay = 1;
  if (network_get_local_ip() < 0) network_okay = 0;
}

void network_shutdown()
{
  network_okay = 0;
}
#endif

int network_start_server()
{
  struct sockaddr_in n;
  int port;
  int i;



  n.sin_family = AF_INET;
  n.sin_addr.s_addr = htonl(INADDR_ANY);
  
  for (i=0; i<N_BLOCKIT_PORTS; i++)
    {
      port = blockit_port[i];
      n.sin_port = htons(port);
      if(bind(server_socket, (struct sockaddr *)&n, sizeof(n)) < 0)
	{
	  if (i == (N_BLOCKIT_PORTS - 1))
	    {
	      closesocket(server_socket);
	      server_socket = 0;
	      return -1;
	    }
	}
      else
	{
	  break;
	}
    }
  
  if(listen(server_socket, 1) < 0)
    {
      closesocket(server_socket); server_socket = 0; return -1;
    }
  
  FD_ZERO(&socks);
  FD_SET(server_socket, &socks);
  return port;
}

int network_check_connect()
{
  fd_set x, y;
  struct timeval t;
  struct sockaddr_in client;
  int size;

  x = socks;
  y = socks;
  t.tv_sec = 0;
  t.tv_usec = 0;
  if (select(server_socket+1, &x, NULL, NULL, &t) < 0)
    return 0;
  if (!FD_ISSET(server_socket, &x)) return 0;

  size = sizeof(client);
  game_socket = accept(server_socket, (struct sockaddr *)&client, &size);
  if (game_socket == BAD_SOCK)
    {
      return -1;
    }

  FD_ZERO(&socks);
  FD_SET(game_socket, &socks);
  return 1;
}

int network_server_handshake()
{
  fd_set x, y;
  struct timeval t;
  char buf[12];

  /* Send magic number across; do handshaking */
  if (send(game_socket, MAGIC_1, 9, 0) != 9)
    {
      closesocket(game_socket);
      socks = y;
      return -1;
    }

  x = socks;
  t.tv_sec = 15;
  t.tv_usec = 0;
  if (select(game_socket+1, &x, NULL, NULL, &t) < 0)
    {
      closesocket(game_socket);
      socks = y;
      return -1;
    }
  if (!FD_ISSET(game_socket, &x))
    {
      closesocket(game_socket);
      socks = y;
      return -1;
    }
  memset (buf, 0, 9);
  recv(game_socket, buf, 9, 0);
  if (strncmp(buf, MAGIC_2, 9) != 0)
    {
      closesocket(game_socket);
      socks = y;
      return -1;
    }

  return 1;
}

char network_ip_buffer[4096];

int network_get_local_ip()
{
  char buf[4096];
  struct hostent *ip;

  gethostname(buf, 4000);
  ip = gethostbyname(buf);
  if (ip == NULL)
    {
      sprintf(network_ip_buffer, "unknown");
      return -1;
    }
  else
    {
      sprintf(network_ip_buffer, "%d.%d.%d.%d",
	      (unsigned char)ip->h_addr[0],
	      (unsigned char)ip->h_addr[1],
	      (unsigned char)ip->h_addr[2],
	      (unsigned char)ip->h_addr[3]);
    }
  return 0;
}

char *network_get_enemy_ip()
{
  struct sockaddr_in r;
  int size;
  int ip;

  size = sizeof(r);
  getpeername(game_socket, (struct sockaddr *)&r, &size);
 // return inet_ntoa(r.sin_addr);
}

int network_client_connect(char *server_ip)
{
  struct sockaddr_in r;
  struct hostent *h;
  struct in_addr a;
  int i;
  int port;
  unsigned int saddr;
  char buf[80];
  char *tmp;
  
  // saddr = inet_addr(server_ip);
  /*
  if (saddr == INADDR_NONE)
    {
      h = gethostbyname(server_ip);
      if (h == NULL) return -1;
      sprintf(buf, "%d.%d.%d.%d",
	      (unsigned char)h->h_addr[0],
	      (unsigned char)h->h_addr[1],
	      (unsigned char)h->h_addr[2],
	      (unsigned char)h->h_addr[3]);
      saddr = inet_addr(buf);
    }


  for (i=0; i<N_BLOCKIT_PORTS; i++)
    {
      game_socket = socket(PF_INET, SOCK_STREAM, 0);
      if (game_socket < 0) return -1;

      port = blockit_port[i];
      r.sin_family = AF_INET;
      r.sin_addr.s_addr = saddr;
      r.sin_port = htons(port);
      if (connect(game_socket, (struct sockaddr *)&r, sizeof(r)) < 0)
	{
	  closesocket(game_socket);
	  game_socket = 0;
	  if (i == (N_BLOCKIT_PORTS - 1))
	    {
	      return -1;
	    }
	}
      else
	{
	  break;
	}
    }



  a.s_addr = saddr;
  tmp = inet_ntoa(a);
  strcpy(server_ip, tmp);

  FD_ZERO(&socks);
  FD_SET(game_socket, &socks);
  return port;

*/
}



int network_client_handshake()
{
  struct timeval t;
  char buf[12];
  fd_set x;

  /* Send magic number across; do handshaking */
  x = socks;
  t.tv_sec = 15;
  t.tv_usec = 0;
  if (select(game_socket+1, &x, NULL, NULL, &t) < 0)
    {
      closesocket(game_socket);
      return -1;
    }
  if (!FD_ISSET(game_socket, &x))
    {
      closesocket(game_socket);
      return -1;
    }
  memset (buf, 0, 9);
  recv(game_socket, buf, 9, 0);
  if (strncmp(buf, MAGIC_1, 9) != 0)
    {
      closesocket(game_socket);
      return -1;
    }

  send(game_socket, MAGIC_2, 9, 0);

  return 0;
}

void network_kill_connection()
{
  closesocket(game_socket);
  game_socket = -1;
  closesocket(server_socket);
  server_socket = 0;
}

int network_check_for_move()
{
  fd_set x;
  struct timeval t;

  x = socks;
  t.tv_sec = 0;
  t.tv_usec = 0;
  if (select(game_socket+1, &x, NULL, NULL, &t) < 0)
    {
      return -1; /* Connection is deceased */
    }
  if (!FD_ISSET(game_socket, &x))
    {
      return 0;
    }
  return 1;
}

int network_send_move(int meta, int x, int y)
{
  unsigned char buf[12];
  fd_set q;
  struct timeval t;

  strcpy(buf, MAGIC_3);
  buf[7] = meta;
  buf[8] = x;
  buf[9] = y;
  if (send(game_socket, buf, 10, 0) != 10) return -1;

#if 0
  q = socks;
  t.tv_sec = 10;
  t.tv_usec = 0;
  if (select(game_socket+1, &q, NULL, NULL, &t) < 0)
    {
      return -1;
    }
  if (!FD_ISSET(game_socket, &q))
    {
      return -1;
    }
  memset(buf, 0, 12);
  recv(game_socket, buf, 8, 0);
  if ((strncmp(buf, MAGIC_4, 8) != 0))
    {
      return -1;
    }
#endif

  return 0;
}

int network_get_move(int *meta, int *x, int *y)
{
  unsigned char buf[12];

  if(recv(game_socket, buf, 10, 0) != 10)
    {
      return -1;
    }
  if (strncmp(buf, MAGIC_3, 7) != 0)
    {
      return -2;
    }
#if 0
  if (send(game_socket, MAGIC_4, 8, 0) != 8)
    {
      return -1;
    }
#endif
  *x = (int)buf[8];
  *y = (int)buf[9];
  *meta = (int)buf[7];

  return 0;
}

int network_send_text(char *t)
{
  char buf[90];

  memset(buf, 0, 76);
  strcpy(buf, MAGIC_5);
  strcat(buf, t);
  if (send(game_socket, buf, 76, 0) != 76) return -1;

  return 0;
}

int network_get_text(char *t)
{
  unsigned char buf[80];

  if(recv(game_socket, buf, 4, 0) != 4)
    {
      return -1;
    }
  if (strncmp(buf, MAGIC_5, 4) != 0)
    {
      if (strncmp(buf, MAGIC_6, 4) != 0)
	{
	  return -1;
	}
      else
	{
	  return 1;
	}
    }
  if(recv(game_socket, t, 72, 0) != 72)
    {
      return -1;
    }

  return 0;
}

int network_end_talk()
{
  if (send(game_socket, MAGIC_6, 4, 0) != 4) return -1;

  return 0;
}

/* This code is a mess.  _Really_ a mess. */
#define SYNC_CHAR_1 244
#define SYNC_CHAR_2 233
#define SYNC_CHAR_3 222

int network_synchronise()
{
  int enemy_sync = 0, send_ack = 1;
  unsigned char buf[80];
  fd_set p, q;
  struct timeval t;
  int t1, t2;
  
 re_sync:
  p = socks;
  t.tv_sec = 0;
  t.tv_usec = 0;
  if (select(game_socket+1, &p, NULL, NULL, &t) < 0)
    {
      return -1;
    }
  if (FD_ISSET(game_socket, &p))
    {
      while (network_check_for_move() == 1)
	{
	  if (recv(game_socket, buf, 1, 0) < 1) return -1;
	  if ((buf[0] == SYNC_CHAR_1) && send_ack)
	    {
	      buf[0] = SYNC_CHAR_1;
	      send(game_socket, buf, 1, 0);
	      buf[0] = SYNC_CHAR_2;
	      send(game_socket, buf, 1, 0);
	      send_ack = 0;
	      enemy_sync++;
	    }
	  if (buf[0] == SYNC_CHAR_2)
	    {
	      enemy_sync++;
	    }
	}
    }
  else
    {
      memset(buf, SYNC_CHAR_1, 76);
      if (send(game_socket, buf, 76, 0) < 0) return -1;
    }

  if (enemy_sync)
    {
      while (network_check_for_move() == 1)
	{
	  if (recv(game_socket, buf, 1, 0) < 1) return -1;
	  if (buf[0] == SYNC_CHAR_1)
	    {
	      enemy_sync = 0;
	      goto re_sync;
	    }
	  if (buf[0] == SYNC_CHAR_3)
	    {
	      buf[0] = '!';
	      send(game_socket, buf, 1, 0);
	      return 1;
	    }
	}

      buf[0] = SYNC_CHAR_3;
      send(game_socket, buf, 1, 0);
      t1 = SDL_GetTicks()+2000;
      while (network_check_for_move() == 0)
	{
	  if (SDL_GetTicks() > t1)
	    {
	      return 0;
	    }
	}
      recv(game_socket, buf, 1, 0);
      if (buf[0] == SYNC_CHAR_3)
	{
	  buf[0] = '!';
	  send(game_socket, buf, 1, 0);
	  return 1;
	}
      if ((buf[0] == '!'))
	{
	  return 1;
	}
    }


  return 0;
}

