#include <SDL.h>
#include "sdlhelp.h"
#include "game.h"
#include "blockit.h"
#include "blockitnet.h"
#include "rungame.h"

#define R_STARTPLAYER GP_BLUE     /* Blue to start. */
static int cur_turn = R_STARTPLAYER;
static int r_net_type = 0;
static int cheat_on = 1; /* only for testing, okay? */

static char r_conv[73];  /* red's messages */
static char b_conv[73];  /* blue's messages */

static void shuffle_message(int c, int t)
{
  int i;
  char *a;

  if (t == GP_RED)
    {
      a = &r_conv[0];
    }
  else
    {
      a = &b_conv[0];
    }

  for (i=0; i<71; i++)
    {
      a[i] = a[i+1];
    }
  a[71] = c;
}

static void erase_message(int t)
{
  int i;
  char *a;

  if (t == GP_RED)
    {
      a = &r_conv[0];
    }
  else
    {
      a = &b_conv[0];
    }

  for (i=71; i>0; i--)
    {
      a[i] = a[i-1];
    }
  a[0] = ' ';
}

void r_start_game(int red, int blue)
{
  int doneflag = 0;
  int redraw = 1;
  int black, white;
  int sqx, sqy, sqr, sqb;
  int meta;
  int winner;
  int x;
  SDL_Event ev;
  SDL_Surface *buf1, *buf2;
  int net_type = 0, tmp;
  int is_human_move, is_net_move;
  int enemy_paused = 0;
  int BLOCK_SIZE = (G_WIDTH==24)?20:32;

  memset (r_conv, ' ', 72);
  memset (b_conv, ' ', 72);
  r_conv[72] = 0; b_conv[72] = 0;

  black = SDL_MapRGB(bl_screen->format, 0x00, 0x00, 0x00);
  white = SDL_MapRGB(bl_screen->format, 0xFF, 0xFF, 0xFF);
  cur_turn = R_STARTPLAYER;

  if (blue == R_NET_MASTER) net_type = R_NET_MASTER;
  if (blue == R_NET_SLAVE) net_type = R_NET_SLAVE;
  if ((blue == R_NET_MASTER)||(blue == R_NET_SLAVE))
    {
      blue = R_NET_SLAVE;
      red = R_NET_MASTER;
    }
  if ((red == R_NET_SLAVE)||(red == R_NET_MASTER))
    {
      blue = R_NET_SLAVE;
      red = R_NET_MASTER;
    }

  r_net_type = net_type;

  g_start_game();

  buf1 = SDL_CreateRGBSurface(0, 640, 480, 16,
			      bl_screen->format->Rmask, bl_screen->format->Gmask,
			      bl_screen->format->Bmask, bl_screen->format->Amask);
  SDL_BlitSurface(bl_screen, NULL, buf1, NULL);

  r_redraw(0);
  buf2 = SDL_CreateRGBSurface(0, 640, 480, 16,
			      bl_screen->format->Rmask, bl_screen->format->Gmask,
			      bl_screen->format->Bmask, bl_screen->format->Amask);
  SDL_BlitSurface(bl_screen, NULL, buf2, NULL);

  SDL_BlitSurface(buf1, NULL, bl_screen, NULL);
  SDL_UpdateRect(bl_screen, 0, 0, 0, 0);

  SDLhelp_CrossFade(bl_screen, buf2, 640, 480);

  SDL_FreeSurface(buf1);
  SDL_FreeSurface(buf2);

  srand(time(NULL));

  for(;;)
    {
      if (redraw)
	{
	  r_redraw(1);
	  redraw = 0;
	}
      if (cur_turn == GP_RED)
	{
	  if ((red > R_HUMAN) && (red < R_NET_MASTER))
	    {
	      ai_move(cur_turn, red);
	      cur_turn = GP_BLUE;
	      redraw = 1;
	      continue;
	    }
	}
      else if (cur_turn == GP_BLUE)
	{
	  if ((blue > R_HUMAN) && (blue < R_NET_MASTER))
	    {
	      ai_move(cur_turn, blue);
	      cur_turn = GP_RED;
	      redraw = 1;
	      continue;
	    }
	}

      /* Check for moves on the other end of the pipe */
      if (net_type)
	{
	  tmp = network_check_for_move();
	  if (tmp == -1) { doneflag = 1; break; }
	  if (tmp == 1)
	    {
	      if (network_get_move(&meta, &sqx, &sqy) < 0)
		{
		  doneflag = 1;
		  break;
		}
	      redraw = 1;
	      if (meta == META_SENDCHAR)
		{
#if 0
		  if (sqy == 255)
		    erase_message((net_type == R_NET_SLAVE)?GP_RED:GP_BLUE);
		  else
		    shuffle_message(sqy, (net_type == R_NET_SLAVE)?GP_RED:GP_BLUE);
#endif
		  network_get_text((net_type==R_NET_SLAVE)?r_conv:b_conv);
		  redraw = 1;
		  continue;
		}
	      if ((meta == META_ENDGAME))
		{
		  /* End of game */
		  net_type = 0; doneflag = 1; break;
		}
	      if ((meta == META_CHATMODE))
		{
		  x = r_chat_mode(net_type);
		  if (x == -1) return;
		  redraw = 1;
		  continue;
		}
	      if (((cur_turn == GP_RED) && (net_type == R_NET_MASTER)) ||
		  ((cur_turn == GP_BLUE)&& (net_type == R_NET_SLAVE)))
		{
		  SDLhelp_Problem(bl_screen, "Enemy tried to place stone in your turn!");
		  continue;
		}
	      if (!g_can_move(cur_turn, sqx, sqy))
		{
		  SDLhelp_Problem(bl_screen, "Warning: opponent is cheating.");
		  continue; // ignore the move
		}
	      r_flash_move(sqx, sqy);
	      if (cur_turn == GP_RED)
		cur_turn = GP_BLUE;
	      else
		cur_turn = GP_RED;
	    }
	}

      if (!SDL_PollEvent(&ev))
	{
	  ev.type = SDL_NOEVENT;
	}

      if (ev.type == SDL_QUIT)
	{
	  doneflag = 1;
	  break;
	}
      if (ev.type == SDL_KEYUP) /* ESC key ==> Quit */
	{
	  if(ev.key.keysym.sym == SDLK_ESCAPE)
	    {
	      doneflag = 1;
	    }
	  if((net_type) && (ev.key.keysym.sym == SDLK_F4))
	    {
	      if(network_send_move(META_CHATMODE, 0, 0) == -1) break;
	      x = r_chat_mode(net_type);
	      if (x == -1) return;
	      redraw = 1;
	      continue;
	    }
	}
      if ((ev.type == SDL_KEYDOWN) && net_type)
	{
	  if (ev.key.keysym.sym == SDLK_BACKSPACE)
	    {
	      erase_message((net_type==R_NET_MASTER)?GP_RED:GP_BLUE);
	      redraw  = 1; /* FIXME: only redraw the message area */
	      continue;
	    }
	  if (ev.key.keysym.sym == SDLK_RETURN)
	    {
	      network_send_move(META_SENDCHAR, 0, 0);
	      network_send_text((net_type==R_NET_SLAVE)?b_conv:r_conv);
	    }
	  if (ev.key.keysym.sym < 127) /* May be printable */
	    {
	      x = ev.key.keysym.sym;
	      if (isprint(x))
		{
		  x = r_shifted_keystroke(x);
		  shuffle_message(x,
				  (net_type==R_NET_MASTER)?GP_RED:GP_BLUE);
		  redraw = 1;
		  continue;
		}
	    }
	}
      if (ev.type == SDL_MOUSEBUTTONDOWN)
	{
	  if ((ev.button.x >= 512) && (ev.button.x <= 640)) /* Correct X coord */
	    {
	      if ((ev.button.y <= 32))
		{
		  doneflag = 1;
		}
	    }

	  /* Check for human moves */
	  is_human_move = ((cur_turn == GP_RED) && (red == R_HUMAN)) ||
	    ((cur_turn == GP_BLUE)&& (blue== R_HUMAN));
	  if (net_type)
	    is_net_move = ((cur_turn == GP_RED) && (red == net_type)) ||
	      ((cur_turn == GP_BLUE) && (blue == net_type));
	  else
	    is_net_move = 0;
	  if (is_human_move || is_net_move || ((SDL_GetModState()&KMOD_ALT)&&cheat_on))
	    {
	      if ((ev.button.x >= 80) && (ev.button.x <= 560) &&
		  (ev.button.y >= 80) && (ev.button.y <= 400))
		{
		  sqx = (ev.button.x - 80) / BLOCK_SIZE;
		  sqy = (ev.button.y - 80) / BLOCK_SIZE;
		  if (g_can_move(cur_turn, sqx, sqy))
		    {
		      if (net_type)
			{
			  if(network_send_move(META_SENDMOVE, sqx, sqy) == -1) { doneflag = 1; break; }
			}
		      r_flash_move(sqx, sqy);
		      if (cur_turn == GP_RED)
			cur_turn = GP_BLUE;
		      else
			cur_turn = GP_RED;
		    }
		  redraw = 1;
		}
	    }
	}

      if ((winner = g_has_finished()))
	{
	  r_finish_game(winner);
	  doneflag = 1;
	}

      if (doneflag) break;
    }

  if (net_type) network_send_move(META_ENDGAME, 0, 0);
  /* Caller takes care of fading; see redraw==2 in bl_title_screen() */
}

int r_shifted_keystroke(int c)
{
  int shiftmode  = (SDL_GetModState()&(KMOD_RSHIFT|KMOD_LSHIFT|KMOD_CAPS))?1:0;

  if (shiftmode)
    {
      if (isalpha(c))
	{
	  c = toupper(c);
	}
      switch (c)
	{
	case '`': c = '~'; break;
	case '-': c = '_'; break;
	case '=': c = '+'; break;
	case '[': c = '{'; break;
	case ']': c = '}'; break;
	case '\\': c = '|'; break;
	case ';': c = ':'; break;
	case '\'': c = '\"'; break;
	case ',': c = '<'; break;
	case '.': c = '>'; break;
	case '/': c = '?'; break;
	  
	case '1': c = '!'; break;
	case '2': c = '@'; break;
	case '3': c = '#'; break;
	case '4': c = '$'; break;
	case '5': c = '%'; break;
	case '6': c = '^'; break;
	case '7': c = '&'; break;
	case '8': c = '*'; break;
	case '9': c = '('; break;
	case '0': c = ')'; break;
	}
    }
  return c;
}

/* Draw a single square. */
static void redraw_square(int update, int i, int j)
{
  SDL_Rect re;
  int BLOCK_SIZE = (G_WIDTH==24)?20:32;

  re.x = i*BLOCK_SIZE + 80;
  re.y = j*BLOCK_SIZE + 80;
  re.w = re.h = BLOCK_SIZE;
  
  if (G_WIDTH == 24)
    {
      switch (g_grid[i][j])
	{
	case 0: /* T Neutral */
	  SDL_BlitSurface(bl_bmp_blocks[0], NULL, bl_screen, &re);
	  break;
	case 1: /* T Red */
	  SDL_BlitSurface(bl_bmp_blocks[1], NULL, bl_screen, &re);
	  break;
	case 2: /* T Blue */
	  SDL_BlitSurface(bl_bmp_blocks[2], NULL, bl_screen, &re);
	  break;
	case 3: /* T Shared */
	  SDL_BlitSurface(bl_bmp_blocks[3], NULL, bl_screen, &re);
	  break;
	case 4: /* S Neutral */
	  SDL_BlitSurface(bl_bmp_blocks[4], NULL, bl_screen, &re);
	  break;
	case 5: /* S Red */
	  SDL_BlitSurface(bl_bmp_blocks[5], NULL, bl_screen, &re);
	  break;
	case 6: /* S Blue */
	  SDL_BlitSurface(bl_bmp_blocks[6], NULL, bl_screen, &re);
	  break;
	case 7: /* S Shared */
	  SDL_BlitSurface(bl_bmp_blocks[7], NULL, bl_screen, &re);
	  break;
	default: /* Can't happen */
	  SDL_Quit();
	}
    }
  else
    {
      switch (g_grid[i][j])
	{
	case 0: /* T Neutral */
	  SDL_BlitSurface(bl_bmp_bblocks[0], NULL, bl_screen, &re);
	  break;
	case 1: /* T Red */
	  SDL_BlitSurface(bl_bmp_bblocks[1], NULL, bl_screen, &re);
	  break;
	case 2: /* T Blue */
	  SDL_BlitSurface(bl_bmp_bblocks[2], NULL, bl_screen, &re);
	  break;
	case 3: /* T Shared */
	  SDL_BlitSurface(bl_bmp_bblocks[3], NULL, bl_screen, &re);
	  break;
	case 4: /* S Neutral */
	  SDL_BlitSurface(bl_bmp_bblocks[4], NULL, bl_screen, &re);
	  break;
	case 5: /* S Red */
	  SDL_BlitSurface(bl_bmp_bblocks[5], NULL, bl_screen, &re);
	  break;
	case 6: /* S Blue */
	  SDL_BlitSurface(bl_bmp_bblocks[6], NULL, bl_screen, &re);
	  break;
	case 7: /* S Shared */
	  SDL_BlitSurface(bl_bmp_bblocks[7], NULL, bl_screen, &re);
	  break;
	default: /* Can't happen */
	  SDL_Quit();
	}
    }

  if (update)
    {
      SDL_UpdateRect(bl_screen, re.x, re.y, re.w, re.h);
    }
}

/* SDL_UpdateRect() will be called if n!=0 */
void r_redraw(int n)
{
  SDL_Rect re;
  int black, white, red, blue, lred, lblue;
  int s_red, s_blue, s_neutral;
  int i,j;
  int BLOCK_SIZE = (G_WIDTH==24)?20:32;
  char buf[80];

  g_get_territory(&s_neutral, &s_red, &s_blue);

  black = SDL_MapRGB(bl_screen->format, 0x00, 0x00, 0x00);
  white = SDL_MapRGB(bl_screen->format, 0xFF, 0xFF, 0xFF);
  red   = SDL_MapRGB(bl_screen->format, 0x80, 0x00, 0x00);
  blue  = SDL_MapRGB(bl_screen->format, 0x00, 0x00, 0x80);
  lred  = SDL_MapRGB(bl_screen->format, 0xFF, 0x80, 0x00);
  lblue = SDL_MapRGB(bl_screen->format, 0x00, 0x80, 0xFF);

  if (cur_turn != -1)
    {
      if (cur_turn == GP_RED)
	r_fade_screen(1, 0, 0);
      else
	r_fade_screen(0, 0, 1);
    }
  else
    {
      if (s_red > s_blue)
	r_fade_screen(1, 0, 0);
      if (s_blue > s_red)
	r_fade_screen(0, 0, 1);
      if (s_blue == s_red)
	r_fade_screen(0, 1, 0);
    }

  /* Button bars */
  re.x = 0; re.y = 0; re.w = 512; re.h = 32;
  SDL_BlitSurface(bl_bmp_sidebar_large, NULL, bl_screen, &re);
  re.x = 512; re.y = 0; re.w = 128; re.h = 32;
  SDL_BlitSurface(bl_bmp_sidebar_small, NULL, bl_screen, &re);

  re.x = 0; re.y = 448; re.w = 512; re.h = 32;
  SDL_BlitSurface(bl_bmp_sidebar_large, NULL, bl_screen, &re);
  re.x = 512; re.y = 448; re.w = 128; re.h = 32;
  SDL_BlitSurface(bl_bmp_sidebar_small, NULL, bl_screen, &re);

  if (cur_turn != -1)
    SDLhelp_DrawString(bl_screen, 540, 8, "Stop game", black);
  else
    SDLhelp_DrawString(bl_screen, 552, 8, "Finish", black);

  /* Display the score */
  sprintf(buf, "%.3d", s_red);
  SDLhelp_DrawString(bl_screen, 540, 456, buf, red);
  SDLhelp_DrawString(bl_screen, 572, 456, ":", black);
  sprintf(buf, "%.3d", s_blue);
  SDLhelp_DrawString(bl_screen, 588, 456, buf, blue);

  /* Say who is winning/has won */
  if (cur_turn != -1)
    {
      if (s_red > s_blue)
	{
	  SDLhelp_DrawString(bl_screen, 8, 456, "RED is winning", red);
	}
      else if (s_blue > s_red)
	{
	  SDLhelp_DrawString(bl_screen, 8, 456, "BLUE is winning", blue);
	}
      else
	{
	  SDLhelp_DrawString(bl_screen, 8, 456, "Equal territory", black);
	}
    }
  else
    {
      if (s_red > s_blue)
	{
	  SDLhelp_DrawString(bl_screen, 8, 456, "RED has won.", red);
	}
      else if (s_blue > s_red)
	{
	  SDLhelp_DrawString(bl_screen, 8, 456, "BLUE has won.", blue);
	}
      else
	{
	  SDLhelp_DrawString(bl_screen, 8, 456, "There is no winner.", black);
	}
    }

  if (r_net_type)
    {
      SDLhelp_DrawString(bl_screen, 9, 41, r_conv, lred);
      SDLhelp_DrawString(bl_screen, 9, 57, b_conv, lblue);
      SDLhelp_DrawString(bl_screen, 8, 40, r_conv, white);
      SDLhelp_DrawString(bl_screen, 8, 56, b_conv, white);
    }

  /* Say who will move next */
  if (cur_turn != -1)
    {
      if (r_net_type == 0)
	{
	  if (cur_turn == GP_RED)
	    SDLhelp_DrawString(bl_screen, 8, 8, "RED to move", red);
	  else
	    SDLhelp_DrawString(bl_screen, 8, 8, "BLUE to move", blue);
	}
      else
	{
	  if ( ((cur_turn == GP_RED) && (r_net_type == R_NET_MASTER)) ||
	       ((cur_turn == GP_BLUE)&& (r_net_type == R_NET_SLAVE))  )
	    SDLhelp_DrawString(bl_screen, 8, 8, "Your move", (cur_turn == GP_RED)?red:blue);
	  else
	    SDLhelp_DrawString(bl_screen, 8, 8, "Waiting for opponent to move",
			       (cur_turn == GP_RED)?red:blue);
	}
    }
  else
    SDLhelp_DrawString(bl_screen, 8, 8, "Game over.", black);

  /* Draw the blocks */
  for (i=0; i<G_WIDTH; i++)
    {
      for (j=0; j<G_HEIGHT; j++)
	{
	  redraw_square(0, i, j);
	}
    }

  /* Finally, update the screen */
  if (n)
    SDL_UpdateRect(bl_screen, 0, 0, 0, 0);
}


void r_finish_game(int winner)
{
  SDL_Surface *buf1, *buf2;
  char buf[100];
  SDL_Rect re;
  SDL_Event ev;
  int black, white;
  int pr, pb, tr, tb, tn;
  int i, j;

  black = SDL_MapRGB(bl_screen->format, 0x00, 0x00, 0x00);
  white = SDL_MapRGB(bl_screen->format, 0xFF, 0xFF, 0xFF);

  cur_turn = -1;

  /* Get this stuff before the board is obliterated. */
  g_get_territory(&tn, &tr, &tb);
  g_get_stones(&pr, &pb);

  /* First, we show the board being filled up. */
  SDL_ShowCursor(0);
  SDLhelp_Delay(200);
  for (j=0; j<G_HEIGHT; j++)
    {
      for (i=0; i<G_WIDTH; i++)
	{
	  g_grid[i][j] |= GT_STONE; /* Add a stone. */
	  redraw_square(1, i, j);
	  SDLhelp_Delay(50);
	}
    }
  r_redraw(1);
  SDLhelp_Delay(500);
  SDL_ShowCursor(1);
  for (;;) /* Wait for key/mouse press */
    {
      SDL_WaitEvent(&ev);
      if ((ev.type == SDL_MOUSEBUTTONDOWN) || (ev.type == SDL_KEYUP)) break;
    }

  cur_turn = GP_NEUTRAL;
  if (winner == GP_RED)
    {
      buf2 = bl_bmp_win_red;
    }
  if (winner == GP_BLUE)
    {
      buf2 = bl_bmp_win_blue;
    }
  if (winner == GP_SHARED)
    {
      buf2 = bl_bmp_win_none;
    }
  
  /* Draw up game-over screen. */
  buf1 = SDL_CreateRGBSurface(0, 640, 480, 16,
			      bl_screen->format->Rmask, bl_screen->format->Gmask,
			      bl_screen->format->Bmask, bl_screen->format->Amask);
  SDL_FillRect(buf1, NULL, black);
  re.x = 320 - (buf2->w>>1);
  re.y = 32;
  re.w = buf2->w;
  re.h = buf2->h;
  SDL_BlitSurface(buf2, NULL, buf1, &re);
  SDLhelp_DrawString(buf1, 320-36, 8, "GAME OVER", white);
  sprintf(buf, "        PIECES OWNED        BOARD COVERAGE");
  SDLhelp_DrawString(buf1, 320-144, 160+0, buf, white);
  sprintf(buf, "Red         %3d                 %.1f%%", pr, ((double)tr/(G_WIDTH*G_HEIGHT))*100);
  SDLhelp_DrawString(buf1, 320-144, 160+16, buf, white);
  sprintf(buf, "Blue        %3d                 %.1f%%", pb, ((double)tb/(G_WIDTH*G_HEIGHT))*100);
  SDLhelp_DrawString(buf1, 320-144, 160+32, buf, white);
  re.x = 160;
  re.y = 400;
  re.w = 320;
  re.h = 40;
  SDL_BlitSurface(bl_bmp_sidebar_button, NULL, buf1, &re);
  SDLhelp_DrawString(buf1, 320-44, 412, "Finish game", black);

  SDLhelp_CrossFade(bl_screen, buf1, 640, 480);

  for (;;)
    {
      SDL_WaitEvent(&ev);
      if ((ev.type == SDL_MOUSEBUTTONDOWN) || (ev.type == SDL_KEYUP)) break;
    }
}


/* ****** 16-bit dependency ****** */
void r_fade_screen(double r, double g, double b)
{
  double rr, gg, bb;
  int i,j;
  Uint32 col;
  Uint16 *vbuf;
  
  if (!bl_ugly)
    {
      for (i=0; i<480; i++)
	{
	  rr = r/480.0 * (double)i * 255.0;
	  gg = g/480.0 * (double)i * 255.0;
	  bb = b/480.0 * (double)i * 255.0;
	  col = SDL_MapRGB(bl_screen->format, rr, gg, bb);
	  vbuf = &(((Uint16 *)bl_screen->pixels)[(i*bl_screen->pitch)>>1]);
	  for (j=0; j<640; j++)
	    vbuf[j] = col;
	}
    }
  else
    {
      col = SDL_MapRGB(bl_screen->format, r*128, g*128, b*128);
      SDL_FillRect(bl_screen, NULL, col);
    }
}

void r_flash_move(int x, int y)
{
  int save_pos;
  int new_pos;
  int i;
  char buf[512];

  if (!g_can_move(cur_turn, x, y))
    {
      sprintf(buf, "Illegal move (can't happen) at (%d,%d)", x, y);
      SDLhelp_Problem(bl_screen, buf);
    }

  save_pos = g_grid[x][y];

  new_pos = GT_STONE | save_pos;

  for (i=0; i<2; i++) /* Flash */
    {
      g_grid[x][y] = new_pos;
      /* r_redraw(1); */
      redraw_square(1, x, y);
      SDLhelp_Delay(200);

      g_grid[x][y] = save_pos;
      redraw_square(1, x, y);
      /* r_redraw(1); */
      SDLhelp_Delay(200);
    }

  g_make_move(cur_turn, x, y);
  r_redraw(1);
}
