/* font_pbm2c.c - Extract character data from cropped PBM screen-dump.
   Filter: PBM is read on stdin, C data is written to stdout.
   Technically, the PBM is a PPM, but why bother avoiding confusion?
   
   This entire system was adapted from the font system used in DGen/SDL,
   which used an uglier font but used what was basically the same technique.
   The code is mine, though.
*/

#include <stdio.h>

/* Limitations:  the font _must_ be 8x16.  Sorry, this
 * makes it a lot simpler to write the draw routine.
 */

char pbm_data[768*16];

/* This programme is a kludge, and may not work if the PBM file is
 * structured in the wrong way. Also, it may require Linux/Unix
 * specific quirks to function properly.
 */

/* For testing only: */
/* #define WRITE_ASCII_ART */
#undef WRITE_ASCII_ART

int main()
{
  char pbm_linebuf[160];
  int pbm_scanbuf;
  int pbm_x, pbm_y;
  int x,y,c, i, j;
  int bitmask;
  unsigned char bits;

  fprintf(stderr, "++ Trying to parse PBM...\n");
  scanf("%s", pbm_linebuf);
  if (strcmp(pbm_linebuf, "P3") != 0)
    {
      fprintf(stderr, "NAK! Incorrect PBM magic: `%s'\n", pbm_linebuf);
      exit(1);
    }
  /* Comments are supposed to be allowed here. If any are found, the programme
   * will choke and die.
   */
  scanf("%d %d", &pbm_x, &pbm_y);
  if (pbm_x == 0) /* Probably a comment */
    {
      fprintf(stderr, "NAK! Bad PBM line #2, maybe a comment\n");
      exit(1);
    }
  if ((pbm_x != 768) ||
      (pbm_y != 16))
    {
      fprintf(stderr, "NAK! Bad size for image: %dx%d\n", pbm_x, pbm_y);
      exit(1);
    }

  fprintf(stderr, "++ PBM file found, slurping in pixels...\n");
  for (y=0; y<16; y++)
    {
      for (x=0; x<768; x++)
	{
	  scanf("%d", &pbm_scanbuf);
	  scanf("%d", &pbm_scanbuf);
	  scanf("%d", &pbm_scanbuf); /* Read three colours, ignore first two! */
	  if ((pbm_scanbuf != 255) &&
	      (pbm_scanbuf != 0))
	    {
	      fprintf(stderr, "NAK! Silly PBM file is a bit odd\n");
	      exit(1);
	    }
	  if (pbm_scanbuf == 0)
	    pbm_data[y*768+x] = 1;
	  else
	    pbm_data[y*768+x] = 0;
	}
    }

  fprintf(stderr, "++ Splitting data up and splurting a C file...\n");
#ifndef WRITE_ASCII_ART
  printf("/* font_data.c - Declares SDLhelp_fontdata which contains a heap of bits. */\n\n");
#endif
  for (c=32; c<128; c++)
    {
#ifndef WRITE_ASCII_ART
      printf("static unsigned char __glyph_data_%02X[16] = { ", c);
#else
      printf("\n\nGlyph %02X (`%c'):\n", c, c);
#endif
      for (i=0; i<16; i++)
	{
	  bitmask = 0x80; bits = 0;
	  for (j=0; j<8; j++)
	    {
	      if (pbm_data[i*768+c*8+j-255])
		bits |= bitmask;
#ifdef WRITE_ASCII_ART
	      if (bits & bitmask)
		printf("#");
	      else
		printf(" ");
#endif
	      bitmask >>= 1;
	    }
#ifndef WRITE_ASCII_ART
	  printf("0x%02X", bits);
	  if (i != 15) printf(", ");
#else
	  printf("\n");
#endif
	}
#ifndef WRITE_ASCII_ART
      printf(" };\n");
#endif
    }
#ifndef WRITE_ASCII_ART
  printf("\n\nunsigned char *SDLhelp_fontdata[96] =\n{\n");
  for (c=32; c<128; c++)
    {
      printf("  __glyph_data_%02X,\n", c);
    }
  printf("};\n\n/* EOF */\n");
#else
  printf("\n\n\nEND OF FILE.\n");
#endif

  fprintf(stderr, "++ All done.\n");
}
